Fix %b and %c so that they count per-transfer bytes again.
[rsync/rsync.git] / io.c
diff --git a/io.c b/io.c
index 0707ca2..b400ba2 100644 (file)
--- a/io.c
+++ b/io.c
@@ -76,6 +76,9 @@ int kluge_around_eof = 0;
 int sock_f_in = -1;
 int sock_f_out = -1;
 
+int64 total_data_read = 0;
+int64 total_data_written = 0;
+
 static struct {
        xbuf in, out, msg;
        int in_fd;
@@ -94,7 +97,8 @@ static int write_batch_monitor_in = -1;
 static int write_batch_monitor_out = -1;
 
 static int ff_forward_fd = -1;
-static char ff_lastchar;
+static int ff_reenable_multiplex = -1;
+static char ff_lastchar = '\0';
 static xbuf ff_xb = EMPTY_XBUF;
 #ifdef ICONV_OPTION
 static xbuf iconv_buf = EMPTY_XBUF;
@@ -348,8 +352,8 @@ static void forward_filesfrom_data(void)
                        ff_forward_fd = -1;
                        write_buf(iobuf.out_fd, "\0\0", ff_lastchar ? 2 : 1);
                        free_xbuf(&ff_xb);
-                       if (protocol_version < 31)
-                               io_start_multiplex_out(iobuf.out_fd);
+                       if (ff_reenable_multiplex >= 0)
+                               io_start_multiplex_out(ff_reenable_multiplex);
                }
                return;
        }
@@ -1016,18 +1020,14 @@ int get_hlink_num(void)
  * for recv_file_list() to use. */
 void start_filesfrom_forwarding(int fd)
 {
-       ff_forward_fd = fd;
-       if (protocol_version < 31) {
-               int save_fd = iobuf.out_fd;
-               /* Older protocols send the files-from data w/o packaging it in
-                * multiplexed I/O packets.  To match this, we temporarily turn
-                * off the multiplexing of our output w/o disabling buffering. */
-               assert(OUT_MULTIPLEXED);
-               /* Be extra, extra sure no messages go out before files-from data. */
-               iobuf.msg.pos = iobuf.msg.len = 0;
-               io_end_multiplex_out(False);
-               iobuf.out_fd = save_fd;
+       if (protocol_version < 31 && OUT_MULTIPLEXED) {
+               /* Older protocols send the files-from data w/o packaging
+                * it in multiplexed I/O packets, so temporarily switch
+                * to buffered I/O to match this behavior. */
+               iobuf.msg.pos = iobuf.msg.len = 0; /* Be extra sure no messages go out. */
+               ff_reenable_multiplex = io_end_multiplex_out(MPLX_TO_BUFFERED);
        }
+       ff_forward_fd = fd;
 
        alloc_xbuf(&ff_xb, FILESFROM_BUFLEN);
 }
@@ -1138,7 +1138,7 @@ void read_args(int f_in, char *mod_name, char *buf, size_t bufsiz, int rl_nulls,
        *argv_p = argv;
 }
 
-int io_start_buffering_out(int f_out)
+BOOL io_start_buffering_out(int f_out)
 {
        if (msgs2stderr && DEBUG_GTE(IO, 2))
                rprintf(FINFO, "[%s] io_start_buffering_out(%d)\n", who_am_i(), f_out);
@@ -1154,7 +1154,7 @@ int io_start_buffering_out(int f_out)
                if (iobuf.out_fd == -1)
                        iobuf.out_fd = f_out;
                assert(f_out == iobuf.out_fd);
-               return 0;
+               return False;
        }
 
        iobuf.out.size = IO_BUFFER_SIZE * 2 - 4;
@@ -1164,10 +1164,10 @@ int io_start_buffering_out(int f_out)
        iobuf.out.pos = iobuf.out.len = 0;
        iobuf.out_fd = f_out;
 
-       return 1;
+       return True;
 }
 
-int io_start_buffering_in(int f_in)
+BOOL io_start_buffering_in(int f_in)
 {
        if (msgs2stderr && DEBUG_GTE(IO, 2))
                rprintf(FINFO, "[%s] io_start_buffering_in(%d)\n", who_am_i(), f_in);
@@ -1176,7 +1176,7 @@ int io_start_buffering_in(int f_in)
                if (iobuf.in_fd == -1)
                        iobuf.in_fd = f_in;
                assert(f_in == iobuf.in_fd);
-               return 0;
+               return False;
        }
 
        iobuf.in.size = IO_BUFFER_SIZE;
@@ -1187,14 +1187,14 @@ int io_start_buffering_in(int f_in)
 
        iobuf.in_fd = f_in;
 
-       return 1;
+       return True;
 }
 
 void io_end_buffering_in(BOOL free_buffers)
 {
-       if (DEBUG_GTE(IO, 2)) {
-               rprintf(FINFO, "[%s] io_end_buffering_in(%s)\n",
-                       who_am_i(), free_buffers ? "True" : "False");
+       if (msgs2stderr && DEBUG_GTE(IO, 2)) {
+               rprintf(FINFO, "[%s] io_end_buffering_in(IOBUF_%s_BUFS)\n",
+                       who_am_i(), free_buffers ? "FREE" : "KEEP");
        }
 
        if (free_buffers)
@@ -1207,9 +1207,9 @@ void io_end_buffering_in(BOOL free_buffers)
 
 void io_end_buffering_out(BOOL free_buffers)
 {
-       if (DEBUG_GTE(IO, 2)) {
-               rprintf(FINFO, "[%s] io_end_buffering_out(%s)\n",
-                       who_am_i(), free_buffers ? "True" : "False");
+       if (msgs2stderr && DEBUG_GTE(IO, 2)) {
+               rprintf(FINFO, "[%s] io_end_buffering_out(IOBUF_%s_BUFS)\n",
+                       who_am_i(), free_buffers ? "FREE" : "KEEP");
        }
 
        io_flush(FULL_FLUSH);
@@ -1217,9 +1217,6 @@ void io_end_buffering_out(BOOL free_buffers)
        if (free_buffers) {
                free_xbuf(&iobuf.out);
                free_xbuf(&iobuf.msg);
-       } else {
-               iobuf.out.pos = iobuf.out.len = 0;
-               iobuf.msg.pos = iobuf.msg.len = 0;
        }
 
        iobuf.out_fd = -1;
@@ -1584,6 +1581,7 @@ void read_buf(int f, char *buf, size_t len)
 
        if (!IN_MULTIPLEXED) {
                memcpy(buf, perform_io(len, PIO_INPUT_AND_CONSUME), len);
+               total_data_read += len;
                if (forward_flist_data)
                        write_buf(iobuf.out_fd, buf, len);
          batch_copy:
@@ -1607,6 +1605,7 @@ void read_buf(int f, char *buf, size_t len)
                /* The bytes at the "data" pointer will survive long
                 * enough to make a copy, but not past future I/O. */
                memcpy(buf, data, siz);
+               total_data_read += siz;
 
                if (forward_flist_data)
                        write_buf(iobuf.out_fd, buf, siz);
@@ -1879,6 +1878,7 @@ void write_buf(int f, const char *buf, size_t len)
                memcpy(iobuf.out.buf + pos, buf, len);
 
        iobuf.out.len += len;
+       total_data_written += len;
 
   batch_copy:
        if (f == write_batch_monitor_out)
@@ -2045,11 +2045,11 @@ void io_printf(int fd, const char *format, ...)
 /* Setup for multiplexing a MSG_* stream with the data stream. */
 void io_start_multiplex_out(int fd)
 {
+       io_flush(FULL_FLUSH);
+
        if (msgs2stderr && DEBUG_GTE(IO, 2))
                rprintf(FINFO, "[%s] io_start_multiplex_out(%d)\n", who_am_i(), fd);
 
-       io_flush(FULL_FLUSH);
-
        iobuf.out_empty_len = 4; /* See also OUT_MULTIPLEXED */
        io_start_buffering_out(fd);
 
@@ -2067,28 +2067,40 @@ void io_start_multiplex_in(int fd)
        io_start_buffering_in(fd);
 }
 
-void io_end_multiplex_in(BOOL free_buffers)
+int io_end_multiplex_in(int mode)
 {
-       if (DEBUG_GTE(IO, 2)) {
-               rprintf(FINFO, "[%s] io_end_multiplex_in(%s)\n",
-                       who_am_i(), free_buffers ? "True" : "False");
-       }
+       int ret = iobuf.in_multiplexed ? iobuf.in_fd : -1;
+
+       if (msgs2stderr && DEBUG_GTE(IO, 2))
+               rprintf(FINFO, "[%s] io_end_multiplex_in(mode=%d)\n", who_am_i(), mode);
 
        iobuf.in_multiplexed = False;
-       iobuf.raw_input_ends_before = 0;
-       io_end_buffering_in(free_buffers);
+       if (mode == MPLX_SWITCHING)
+               iobuf.raw_input_ends_before = 0;
+       else
+               assert(iobuf.raw_input_ends_before == 0);
+       if (mode != MPLX_TO_BUFFERED)
+               io_end_buffering_in(mode);
+
+       return ret;
 }
 
-/* Stop output multiplexing. */
-void io_end_multiplex_out(BOOL free_buffers)
+int io_end_multiplex_out(int mode)
 {
-       if (DEBUG_GTE(IO, 2)) {
-               rprintf(FINFO, "[%s] io_end_multiplex_out(%s)\n",
-                       who_am_i(), free_buffers ? "True" : "False");
-       }
+       int ret = iobuf.out_empty_len ? iobuf.out_fd : -1;
+
+       if (msgs2stderr && DEBUG_GTE(IO, 2))
+               rprintf(FINFO, "[%s] io_end_multiplex_out(mode=%d)\n", who_am_i(), mode);
 
-       io_end_buffering_out(free_buffers);
+       if (mode != MPLX_TO_BUFFERED)
+               io_end_buffering_out(mode);
+       else
+               io_flush(FULL_FLUSH);
+
+       iobuf.out.len = 0;
        iobuf.out_empty_len = 0;
+
+       return ret;
 }
 
 void start_write_batch(int fd)